 --
 --
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEEEEEEE
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEE
 --    P               A          A    R R                       S    E
 --    P               A          A    R  R                      S    E
 --    P               A          A    R   R                     S    E
 --    P               A          A    R     R                   S    E
 --    P               A          A    R       R                 S    E
 --    P               A          A    R         R     SSSSSSSSSSS    EEEEEEEEEEEE
 --
 --
 --
 --
 --     Copyright (C)  2002  Daniel Gaudry (Daniel.Gaudry@wanadoo.fr)
 --     9 av calmels
 --     92270 bois colombes  france
 --     Home +33147862334
 --     Cell +33664195009
 --     Work +33148575063
 --     Fax  +33148571106
 --
 --     This library is free software; you can redistribute it and/or
 --     modify it under the terms of the GNU Library General Public
 --     License as published by the Free Software Foundation; either
 --     version 2 of the License, or (at your option) any later version.
 --
 --     This library is distributed in the hope that it will be useful,
 --     but WITHOUT ANY WARRANTY; without even the implied warranty of
 --     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 --     Library General Public License for more details.
 --
 --     You should have received a copy of the GNU Library General Public
 --     License along with this library; if not, write to the Free
 --     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 --     MA 02111-1307, USA
 
 --| version 1.0 january  15th, 2001 original coding
 --| version 1.1 August   05th, 2001 use of rw to load dictionary trees
 
 with Rw;
 with Ada.Strings;
 with Ada.Strings.Fixed;
 with Ada.Strings.Maps;
 with Ada.Strings.Maps.Constants;
 
 with Ada.Text_Io;
 
 package body Ae_200
 is
 --|                         |=============|
 --|                         |  Load_Tree  |
 --|                         |=============|
 --|
 --|
    procedure Load_Tree
    is
    Word_Starts_At         : Integer := 1;
    Dictionary_Tree_Tagged : Dictionary_Tree_Tagged_Type;
    Current_Number         : Integer := 1;
    begin
    --|
    --|load reserved words
    --|
    for R in Rw.All_Reserved_Words'Range loop
       --|
       --|load entry number
       --|
       Dictionary_Tree_Tagged.Number := R;
       --|
       --|load word
       --|
       Poly_Dic_Tree.Add_To_Dictionary(The_Record             => Dictionary_Tree_Tagged,
                                       In_The_Dictionary_Tree => Reserved_Words_Dictionary_Tree,
                                       With_The_Label         => Rw.All_Reserved_Words(R).all);
    end loop;
    --|
    --|load attributes
    --|
    for R in Rw.All_Attributes'Range loop
       --|
       --|load entry number
       --|
       Dictionary_Tree_Tagged.Number := R;
       --|
       --|load word
       --|
       Poly_Dic_Tree.Add_To_Dictionary(The_Record             => Dictionary_Tree_Tagged,
                                       In_The_Dictionary_Tree => Attributes_Dictionary_Tree,
                                       With_The_Label         => Rw.All_Attributes(R).all);
    end loop;
    --|
    --|set flag
    --|
    Tree_Is_Loaded := True;
    end Load_Tree;
 --|
 --|                         |===================|
 --|                         |  Is_Reserved_Word |
 --|                         |===================|
 --|
 --|
    function Is_Reserved_Word(The_Word : in     String) return Boolean
    is
    Local_Data_Access : Poly_Dic_Tree.Tagged_Data_Access_Type := null;
    Dummy_Word        : String
    := Ada.Strings.Fixed.Trim(Source => The_Word,
                              Side   => Ada.Strings.Both);
    begin
    --|
    --|   init if needed
    --|
    if not Tree_Is_Loaded
       then
       Load_Tree;
    end if;
    --|
    --|put in lower case for comparison
    --|
    Dummy_Word := Ada.Strings.Fixed.Translate(Source  => Dummy_Word,
                                              Mapping => Ada.Strings.Maps.Constants.Lower_Case_Map);
    --|
    --|  word belongs to the tree ??
    --|
    Poly_Dic_Tree.Get_Record_Access_From_Label(From_The_Dictionary_Tree => Reserved_Words_Dictionary_Tree,
                                               And_The_Label            => Dummy_Word,
                                               Tagged_Data_Access       => Local_Data_Access);
    --|
    --|
    --|       d e b u g   m e s s a g e      
    --|_____________________________________________________________________________
    --|
    --Ada.Text_Io.Put_Line("Is_Reserved_Word"
    --& Dummy_Word
    --& Integer'Image(Dictionary_Tree_Tagged_Type(Local_Data_Access.all).Number));
    --|
    --|
    --|    e n d   d e b u g   m e s s a g e  
    --|_____________________________________________________________________________
    --|
    return Dictionary_Tree_Tagged_Type(Local_Data_Access.all).Number /= 0;
 
    exception
 
    when Poly_Dic_Tree.No_Such_A_Word | Constraint_Error => return False;
 
    end Is_Reserved_Word;
 --|
 --|                         |===============|
 --|                         |  Is_Attribute |
 --|                         |===============|
 --|
 --|
    function Is_Attribute(The_Word : in     String) return Boolean
    is
    Local_Data_Access : Poly_Dic_Tree.Tagged_Data_Access_Type := null;
    Dummy_Word        : String
    := Ada.Strings.Fixed.Trim(Source => The_Word,
                              Side   => Ada.Strings.Both);
 
    begin
    --|
    --|   init if needed
    --|
    if not Tree_Is_Loaded
       then
       Load_Tree;
    end if;
    --|
    --|put in lower case for comparison
    --|
    Dummy_Word := Ada.Strings.Fixed.Translate(Source  => Dummy_Word,
                                              Mapping => Ada.Strings.Maps.Constants.Lower_Case_Map);
    --|
    --|  word belongs to the dictionary tree ??
    --|
    Poly_Dic_Tree.Get_Record_Access_From_Label(From_The_Dictionary_Tree => Attributes_Dictionary_Tree,
                                               And_The_Label            => Dummy_Word,
                                               Tagged_Data_Access       => Local_Data_Access);
 
    --|
    --|
    --|       d e b u g   m e s s a g e      
    --|_____________________________________________________________________________
    --|
    --Ada.Text_Io.Put_Line("Is_Attribute"
    --& Dummy_Word
    --& Integer'Image(Dictionary_Tree_Tagged_Type(Local_Data_Access.all).Number));
    --|
    --|
    --|    e n d   d e b u g   m e s s a g e  
    --|_____________________________________________________________________________
    --|
    return Dictionary_Tree_Tagged_Type(Local_Data_Access.all).Number /= 0;
 
    exception
 
    when Poly_Dic_Tree.No_Such_A_Word | Constraint_Error => return False;
 
    end Is_Attribute;
 --|
 --|              ||
 --|              |           Number_Of          |
 --|              |______________________________|
 --|
    function Number_Of(The_Word : in     String) return Integer
    is
    Result            : Integer                               := 0;
    Local_Data_Access : Poly_Dic_Tree.Tagged_Data_Access_Type := null;
    Dummy_Word        : String
    := Ada.Strings.Fixed.Trim(Source => The_Word,
                              Side   => Ada.Strings.Both);
    begin
    --|
    --|   init if needed
    --|
    if not Tree_Is_Loaded
       then
       Load_Tree;
    end if;
    --|
    --|put in lower case for comparison
    --|
    Dummy_Word := Ada.Strings.Fixed.Translate(Source  => Dummy_Word,
                                              Mapping => Ada.Strings.Maps.Constants.Lower_Case_Map);
    --|
    --|  word belongs to the tree ??
    --|
    Poly_Dic_Tree.Get_Record_Access_From_Label(From_The_Dictionary_Tree => Reserved_Words_Dictionary_Tree,
                                               And_The_Label            => Dummy_Word,
                                               Tagged_Data_Access       => Local_Data_Access);
 
    Result := Dictionary_Tree_Tagged_Type(Local_Data_Access.all).Number;
    return Result;
 
    exception
    when others => return 0;
 
    end Number_Of;
 --|
 --|                         ||
 --|                         | variable |
 --|                         |__________|
 --|
    function Variable(User_Entry : in     String) return Boolean
    is
    --|
    --|declarations
    --|
    Current_Character : Character    := ' ';
    Current_State     : State_Type   := State_Type'First;
    Current_Command   : Command_Type := 1;-- entry stage is 1 !!!!!!!!!!!!!!!!!
    Index             : Integer      := User_Entry'First;
    --|
    --|code begining
    --|
    begin
    --|
    --|  loop on all valid indices of variable
    --|
    while Index in User_Entry'First .. User_Entry'Last loop
       --|
       --|get current character
       --|
       Current_Character := User_Entry(Index);
       --|
       --| translate into state
       --|
       Current_State := Look_Up_Table(Current_Character);
       --|
       --| get current command
       --|
       Current_Command := State_Command_Matrix(Current_State, Current_Command);
       --|
       --|  error  exit
       --|
       exit when Current_Command = 0;
       --|
       --| increment index
       --|
       Index := 1 + Index;
 
    end loop;
    --|
    --|  error ???
    --|
    return not (Current_Command = 0 or else Current_Command = 3);
 
    exception
    when others => return False;
    end Variable;
 
 end Ae_200;
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
