 --
 --
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEEEEEEE
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEE
 --    P               A          A    R R                       S    E
 --    P               A          A    R  R                      S    E
 --    P               A          A    R   R                     S    E
 --    P               A          A    R     R                   S    E
 --    P               A          A    R       R                 S    E
 --    P               A          A    R         R     SSSSSSSSSSS    EEEEEEEEEEEE
 --
 --
 --
 --
 --     Copyright (C)  2002  Daniel Gaudry (Daniel.Gaudry@wanadoo.fr)
 --     9 av calmels
 --     92270 bois colombes  france
 --     Home +33147862334
 --     Cell +33664195009
 --     Work +33148575063
 --     Fax  +33148571106
 --
 --     This library is free software; you can redistribute it and/or
 --     modify it under the terms of the GNU Library General Public
 --     License as published by the Free Software Foundation; either
 --     version 2 of the License, or (at your option) any later version.
 --
 --     This library is distributed in the hope that it will be useful,
 --     but WITHOUT ANY WARRANTY; without even the implied warranty of
 --     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 --     Library General Public License for more details.
 --
 --     You should have received a copy of the GNU Library General Public
 --     License along with this library; if not, write to the Free
 --     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 --     MA 02111-1307, USA
 
 --|
 --| version 1.00 may         02 nd 2000 initial coding
 --| version 1.1 August       08 th 2001 moved reserved words and attributes to rw.ads
 
 with Poly_Dic_Tree;
 
 pragma Elaborate_All(Poly_Dic_Tree);
 
 package Ae_200
 is
 --|
 --|                         ||
 --|                         |  is_reserved_word |
 --|                         |___________________|
 --|
 function Is_Reserved_Word(The_Word : in     String) return Boolean;
 --|
 --|                         ||
 --|                         |  is_attribute |
 --|                         |_______________|
 --|
 --|
 function Is_Attribute(The_Word : in     String) return Boolean;
 --|
 --|              ||
 --|              |           Number_Of          |
 --|              |______________________________|
 --|
 function Number_Of(The_Word : in     String) return Integer;
 --|
 --|                         ||
 --|                         | variable |
 --|                         |__________|
 --|
 function Variable(User_Entry : in     String) return Boolean;
 --|
 --|              ||
 --|              |  P R I V A T E   P A R T     |
 --|              |______________________________|
 --|
 private
 --|
 --|   declarations
 --|
 Tree_Is_Loaded : Boolean := False;
 --|
 --| utilities
 --|
 type Dictionary_Tree_Tagged_Type
 is new Poly_Dic_Tree.Tagged_Data_Type
 with
 record
    Number : Integer := 0;
 end record;
 --|
 --| dictionary tress
 --|
 Reserved_Words_Dictionary_Tree : Poly_Dic_Tree.Dictionary_Tree_Type;
 Attributes_Dictionary_Tree     : Poly_Dic_Tree.Dictionary_Tree_Type;
 --|
 --|
 --|
 --|               V A R I A B L E
 --|
 --| the testing is done using a finite state automaton driven by a state/command table
 --| the states (current character in the line to be tested) are summarized below
 --|
 type State_Type is (Letter,
                     Number,
                     Other_Character,
                     Underscore);
 --|
 --| at this time there are 3 different command columns in the state/command table
 --|
 type Command_Type is new Integer range 0 .. 3;
 --|
 --| define the SATE/COMMAND table
 --|       the state command table contains the next command (column)
 --|
 type State_Command_Matrix_Type is array (State_Type'Range, Command_Type'Range) of Command_Type;
 
 --|     C O L U M N S   U S E
 --|     
 --|
 --|  0 : error exit
 --|  1 : entry letter only
 --|  2 : letter and number back to 2  underscore to 3
 --|  3 : letter and number back to 2
 --|
 State_Command_Matrix : constant State_Command_Matrix_Type
 --S T A T E------------ 0   1   2  3
 := (Letter          => (00, 02, 02, 02),
     Number          => (00, 00, 02, 02),
     Other_Character => (00, 00, 00, 00), -- all to zero not permitted
     Underscore      => (00, 00, 03, 00));
 --|
 --|  we need to translate EFFICIENTLY the characters into states
 --| the only way is a
 --|                        L O O K _ U P   T A B L E
 --|                        
 --|
 type Look_Up_Table_Type is array (Character) of State_Type;
 pragma Pack(Look_Up_Table_Type);
 --|
 --| assign  ***** A L L ***** CHARACTERS to states
 --|
 Look_Up_Table : constant Look_Up_Table_Type := ('0' .. '9'           => Number,
                                                 'a' .. 'z'|'A' .. 'Z'=> Letter,
                                                 '_'                  => Underscore,
                                                 others               => Other_Character);
 
 
 end Ae_200;
 
 
 
 
 
 
 
