 --
 --
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEEEEEEE
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEE
 --    P               A          A    R R                       S    E
 --    P               A          A    R  R                      S    E
 --    P               A          A    R   R                     S    E
 --    P               A          A    R     R                   S    E
 --    P               A          A    R       R                 S    E
 --    P               A          A    R         R     SSSSSSSSSSS    EEEEEEEEEEEE
 --
 --
 --
 --
 --     Copyright (C)  2002  Daniel Gaudry (Daniel.Gaudry@wanadoo.fr)
 --     9 av calmels
 --     92270 bois colombes  france
 --     Home +33147862334
 --     Cell +33664195009
 --     Work +33148575063
 --     Fax  +33148571106
 --
 --     This library is free software; you can redistribute it and/or
 --     modify it under the terms of the GNU Library General Public
 --     License as published by the Free Software Foundation; either
 --     version 2 of the License, or (at your option) any later version.
 --
 --     This library is distributed in the hope that it will be useful,
 --     but WITHOUT ANY WARRANTY; without even the implied warranty of
 --     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 --     Library General Public License for more details.
 --
 --     You should have received a copy of the GNU Library General Public
 --     License along with this library; if not, write to the Free
 --     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 --     MA 02111-1307, USA
 --
 --| version 1.00 may         02 nd 2001 initial coding
 
 package Ae_20_A
 is
 --|
 --|    ||
 --|    | F I N I T E   S T A T E S   A U T O M A T O N   D E F I N I T I O N S  |
 --|    |________________________________________________________________________|
 --|
 
 --|
 --| the parsing is done using a finite Command automaton driven by a Command/STATE table
 --| the Commands (current character in the line to be parsed) are summarized below
 --|
 --| all lines that are the same should be collapsed as only one)
 --| the Commands are letters digits , various separators and miscellaneous characters to be used in strings
 --|  Single_Character_Separator are   | ( ) ; , & althogether
 --|
 type Command_Type is (Letter,
                       Digit,
                       Double_Quote,
                       Sharp,
                       Single_Character_Separator,
                       Colon,
                       Underline,
                       Slash,
                       Minus,
                       Star,
                       Dot,
                       Single_Quote,
                       Less_Than,
                       More_Than,
                       Assign,
                       Space,
                       Miscellaneous,
                       Ee,
                       Extended_Digit,
                       Plus);
 --|
 --| at this time thare are 34 different STATE columns in the Command/STATE table
 --|
 type State_Type is new Integer range 0 .. 34;
 --|
 --|                                 ||
 --|                                 |  STATE/COMMAND table |
 --|                                 |______________________|
 --|
 type Command_State_Matrix_Type is array (State_Type'Range, Command_Type'Range) of State_Type;
 pragma Pack(Command_State_Matrix_Type);
 
 --|
 --|             version 1.00 June 20th, 2001
 --|             version 1.1 June 25th, 2001  added 17 in all error cases
 --|
 --|
 --|
 Command_State_Matrix : constant Command_State_Matrix_Type
 --|
 --|                              ||
 --|                              | START OF FINITE AUTOMATON STATE / COMMAND TABLE  |
 --|                              |__________________________________________________|
 
 --|
 --|
 --|
 --|                               ||
 --|                               |  Start / END STATE   |
 --|                               |______________________|
 
 --|
 --|
 --|
 --|      | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
 --|      | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
 --|      | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
 --|      | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
 --|      |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
 --|      |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
 --|    __|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
 --|      | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
 --|    |||||||||||||||||||||
 := (00 => (01, 18, 06, 17, 03, 09, 17, 13, 14, 16, 08, 04, 12, 11, 10, 34, 17, 01, 01, 03),
     --|
     --|
     --|                               ||
     --|                               | Variable name |
     --|                               |_______________|
     --|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     01 => (01, 01, 00, 00, 00, 00, 02, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 01, 01, 00),
     02 => (01, 01, 00, 00, 00, 00, 17, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 01, 01, 00),
     --|
     --|
     --|                               ||
     --|                               | Separator between tokens:  end of token |
     --|                               |_________________________________________|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     03 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00),
     --|
     --|        ||
     --|        | single quote |
     --|        |______________|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 | 05,4 was 00
     --||||||||||||||||||||||
     04 => (05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05, 05),
     05 => (00, 00, 00, 17, 03, 17, 17, 17, 17, 17, 17, 33, 17, 17, 17, 00, 17, 00, 00, 17), --05,2 Was 17
     --|                                                                17
     --|       ||
     --|       | Double quote |
     --|       |______________|
     --|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     06 => (06, 06, 07, 06, 06, 06, 06, 06, 06, 06, 06, 06, 06, 06, 06, 06, 06, 06, 06, 06),
     07 => (00, 00, 06, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00),
     --|
     --|        ||
     --|        | Dot |
     --|        |_____|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     08 => (00, 00, 00, 17, 00, 17, 17, 17, 17, 17, 03, 17, 17, 17, 17, 00, 17, 00, 00, 17),
     --|
     --|       ||
     --|       | :=  |
     --|       |_____|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     09 => (00, 17, 17, 17, 00, 17, 17, 17, 17, 17, 17, 17, 17, 17, 03, 00, 17, 00, 00, 17),
     --|
     --|       ||
     --|       | => |
     --|       |____|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     10 => (00, 00, 00, 17, 00, 17, 17, 17, 00, 17, 00, 00, 17, 03, 17, 00, 17, 00, 00, 00),
     --|
     --| ||
     --| | >=   and   >>  |
     --| |________________|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     11 => (00, 00, 00, 17, 00, 17, 17, 17, 00, 17, 17, 17, 17, 03, 03, 00, 17, 00, 00, 00),
     --|
     --| ||
     --| | <=    <<   and <> |
     --| |___________________|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     12 => (00, 00, 00, 17, 00, 17, 17, 17, 00, 17, 17, 17, 03, 03, 03, 00, 17, 00, 00, 00),
     --|
     --| ||
     --| | /= |
     --| |____|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     13 => (00, 00, 17, 17, 00, 17, 17, 17, 00, 17, 17, 17, 17, 17, 03, 00, 17, 00, 00, 00),
     --|
     --| ||
     --| | --  |
     --| |_____|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     14 => (00, 00, 00, 00, 00, 17, 17, 17, 15, 17, 17, 00, 17, 17, 17, 00, 17, 00, 00, 17),
     15 => (15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15),
     --|
     --| ||
     --| | ** |
     --| |____|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     16 => (00, 00, 17, 00, 00, 17, 17, 17, 00, 03, 17, 00, 17, 17, 17, 00, 17, 00, 00, 00),
     --|
     --| ||
     --| | Miscellaneous |
     --| |_______________|
     --|
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     17 => (17, 17, 00, 17, 00, 00, 17, 00, 00, 02, 00, 00, 00, 00, 00, 00, 17, 17, 17, 00),
     --
     --
     --                                  ||
     --                                  | NUMBER  (integer or float or hex) |
     --                                  |___________________________________|
     --
     --
     --
     --            digit                         x_digit                           x_digit
     --           |-------|                     |-------|                          |-------|
     --           |       |                     |       |                          |       |
     --  digit    |       V sharp      x_digit  |       V   dot        x_digit     |       V sharp
     --  ------->  18  OK -------> 20 --------->   21    --------> 23 ---------->     24    ---------
     --          / |     ^                      |      ^  \                        |      ^          |
     --         /  |u    |d                     |u     |x  \ s                     |u     |x         |
     --    dot /   |n    |i                     |n     |d   \ h                    |n     |d         |
     --       /    |d    |g                     |d     |i    \ a                   |d     |i         |
     --      /     |     |i                     |      |g     \ r                  |      |g         |
     --      |     |     |t                     |      |i      \ p                 |      |i         |
     --      |     V 19  |                      |      |t       \                  |      |t         |
     --      V                                  V   22 |         \                 V   25 |          |
     --      |                                                    \                                  |
     --      V                                                      \                                |
     --     30                                                       \-->-->-->--------------------->|
     --      |         /--                                                                           V
     --      | d      /t /                                                                         26  OK
     --      | i     /i /                                                                            |
     --      V g    /g /                                                                             | e
     --      | i   /i /                                                                              | E
     --      | t  /d /                                                                               |
     --      V   ^  /                                                                                |
     --         /  V                              eE                                                 V
     --      31 OK ------->---------------->---------->---------->-------->------------->---------> 27
     --   |     ^                                                                                 + /  \ d
     --   |u    |d                                                                                -/    \ i
     --   |n    |i                                                                                /      \ g
     --   |d    |g                                                                               /        \ i
     --   |     |i                                                                               |         | t
     --   |     |t                                                                               V         V
     --   V 32                                                                                 28-------> 29  OK
     --                                                                                            digit   |     ^
     --                                                                                                    |     |
     --                                                                                                    |-----|
     --                                                                                                     digit
     --
     --|  | g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --|  | . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --|  | . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --|  | Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --|  |   |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --|  |   |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|__|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --|  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     18 => (17, 18, 17, 20, 00, 00, 19, 00, 00, 00, 30, 17, 00, 00, 00, 00, 17, 17, 17, 00),
     19 => (17, 18, 17, 17, 00, 00, 17, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 17, 17, 00),
     20 => (17, 21, 17, 17, 00, 00, 00, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 21, 21, 00),
     21 => (17, 21, 17, 26, 00, 00, 22, 00, 00, 00, 23, 17, 00, 00, 00, 00, 17, 21, 21, 00),
     22 => (17, 21, 17, 17, 00, 00, 17, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 21, 21, 00),
     23 => (17, 24, 17, 17, 00, 00, 00, 00, 00, 00, 03, 17, 00, 00, 00, 00, 17, 24, 24, 00),
     24 => (17, 24, 17, 26, 00, 00, 25, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 24, 24, 00),
     25 => (17, 24, 17, 17, 00, 00, 17, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 24, 24, 00),
     26 => (17, 00, 17, 17, 00, 00, 00, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 27, 17, 00),
     27 => (17, 29, 17, 17, 00, 00, 00, 00, 28, 00, 17, 17, 00, 00, 00, 00, 17, 17, 17, 28),
     28 => (17, 29, 17, 17, 00, 00, 00, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 17, 17, 00),
     29 => (17, 29, 17, 17, 00, 00, 00, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 17, 17, 00),
     30 => (17, 31, 17, 17, 00, 00, 00, 00, 00, 00, 03, 17, 00, 00, 00, 00, 17, 17, 17, 00),
     31 => (17, 31, 17, 17, 00, 00, 32, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 27, 17, 00),
     32 => (17, 31, 17, 17, 00, 00, 17, 00, 00, 00, 17, 17, 00, 00, 00, 00, 17, 17, 17, 00),
     --|
     --|                              ||
     --|                              | character |
     --|                              |___________|
     --|
     --| |  g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --| |  . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --| |  . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --| |  Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --| |    |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --| |    |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|_|____|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --| |  0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     33 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00),
     --|
     --|                              ||
     --|                              |   Space   |
     --|                              |___________|
     --|
     --| |  g | 0 |   |   | | |   |   |   |   |   |   |   |   |   |   |   | O |   | a |   |
     --| |  . | . |   |   | ( |   |   |   |   |   |   |   |   |   |   | s | t |   | . |   |
     --| |  . | . |   |   | ) |   |   |   |   |   |   |   |   |   |   | p | h |   | . |   |
     --| |  Z | 9 | " | # | ; |   |   |   |   |   |   |   |   |   |   | a | e |   | d |   |
     --| |    |   |   |   | , |   |   |   |   |   |   |   |   |   |   | c | r | e | F |   |
     --| |    |   |   |   | & | : | _ | / | - | * | . | ' | < | > | = | e | s | E |   | + |
     --|_|____|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|
     --| |  0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |
     --||||||||||||||||||||||
     34 => (01, 18, 06, 17, 03, 09, 17, 13, 14, 16, 08, 04, 12, 11, 10, 34, 17, 01, 01, 03));
 
 --|
 --|  we need to translatee EFFICCIENTLY the characters into Commands :
 --|
 --|                       ||
 --|                       | L O O K _ U P   T A B L E  |
 --|                       |____________________________|
 --|
 type Look_Up_Table_Type is array (0 .. 255) of Command_Type;
 --|
 --| assign  ***** A L L ***** ascii codes to Commands
 --|
 Look_Up_Table : constant
 Look_Up_Table_Type :=
 (103 .. 122 | 71 .. 90                                                         => Letter, -- 00
  48 .. 57                                                                      => Digit, -- 01
  34                                                                            => Double_Quote, -- 02
  35                                                                            => Sharp, -- 03
  124 | 40 | 41 | 59 | 44 | 38                                                  => Single_Character_Separator, -- 04
  58                                                                            => Colon, -- 05
  95                                                                            => Underline, -- 06
  47                                                                            => Slash, -- 07
  45                                                                            => Minus, -- 08
  42                                                                            => Star, -- 09
  46                                                                            => Dot, -- 10
  39                                                                            => Single_Quote, -- 11
  60                                                                            => Less_Than, -- 12
  62                                                                            => More_Than, -- 13
  61                                                                            => Assign, -- 14
  9 .. 13 | 32                                                                  => Space, -- 15
  0 .. 8 | 14 .. 31 | 33 | 36 | 37 | 63 | 64 | 91 .. 94 | 96 | 123 | 125 .. 255 => Miscellaneous, -- 16
  101 | 69                                                                      => Ee, -- 17
  97 .. 100 | 65 .. 68 | 102 | 70                                               => Extended_Digit, -- 18
  43                                                                            => Plus);-- 19
 
 end Ae_20_A;
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
