 --
 --
 --
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEEEEEEE
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEE
 --    P               A          A    R R                       S    E
 --    P               A          A    R  R                      S    E
 --    P               A          A    R   R                     S    E
 --    P               A          A    R     R                   S    E
 --    P               A          A    R       R                 S    E
 --    P               A          A    R         R     SSSSSSSSSSS    EEEEEEEEEEEE
 --
 --
 --
 --
 --     Copyright (C)  2002  Daniel Gaudry (Daniel.Gaudry@wanadoo.fr)
 --     9 av calmels
 --     92270 bois colombes  france
 --     Home +33147862334
 --     Cell +33664195009
 --     Work +33148575063
 --     Fax  +33148571106
 --
 --     This library is free software; you can redistribute it and/or
 --     modify it under the terms of the GNU Library General Public
 --     License as published by the Free Software Foundation; either
 --     version 2 of the License, or (at your option) any later version.
 --
 --     This library is distributed in the hope that it will be useful,
 --     but WITHOUT ANY WARRANTY; without even the implied warranty of
 --     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 --     Library General Public License for more details.
 --
 --     You should have received a copy of the GNU Library General Public
 --     License along with this library; if not, write to the Free
 --     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 --     MA 02111-1307, USA
 --
 --
 -- Version 1.00 March 7 th, 2002
 -- Version 1.1  May  28 th, 2002     split mod and abs from operators
 --
 --
 --
 --
 --
 with Rt;
 with Rw;
 
 package Name_Test
 is
 --|
 --|
 --|
 --|                                 ||
 --|                                 |      Test_Name       |
 --|                                 |______________________|
 --|
 procedure Test_Name(Code_State                 : in out Rt.Code_State_Type;
                     Ok                         :    out Boolean;
                     Skip_Last_Left_Parenthesis : in     Boolean := False);
 
 --|________________
 --|
 --| private
 --|________________
 --|
 --|
 private
 --|________________________________________________________________________________
 --|
 --|          t y p e   d e f i n i t i o n s
 --|________________________________________________________________________________
 --|
 --|
 
 --|
 --|
 --|
 type Command_Type is new Integer range 1 .. 22;
 
 type State_Type is new Integer range 0 .. 34;
 --|
 --|
 --|
 type Truth_Table_Type is array (Integer range <>) of Boolean;
 pragma Pack(Truth_Table_Type);
 --|
 --|
 --|
 type State_Table_Type is array (State_Type range <>) of Boolean;
 pragma Pack(State_Table_Type);
 --|
 --|
 --|
 type Command_Table_Type is array (Command_Type range <>) of Boolean;
 pragma Pack(Command_Table_Type);
 --|
 --|
 --|
 type Look_Up_Table_Type is array (Integer range <>) of Command_Type;
 pragma Pack(Look_Up_Table_Type);
 --|
 --|
 --|
 subtype Look_Up_Table_Index_Type is Integer range 1 .. Rw.First_Free_Index;
 type State_Command_Matrix_Type is array (State_Type range <>, Command_Type range <>) of State_Type;
 pragma Pack(State_Command_Matrix_Type);
 --|
 --|                                 ||
 --|                                 | variables                  |
 --|                                 |____________________________|
 --|
 Current_Code_State    : Rt.Code_State_Type;
 Depth                 : Integer               := 0;
 Miscellaneous_Command : constant Command_Type := 19;
 Error_State           : constant State_Type   := 00;
 Starting_State        : constant State_Type   := 01;
 Ok_State              : constant State_Type   := 02;
 Recursion_State       : constant State_Type   := 03;
 --|
 --|                                 ||
 --|                                 |  L O O K _ U P   T A B L E |
 --|                                 |____________________________|
 --|
 Look_Up_Table : constant Look_Up_Table_Type(0 .. Rw.First_Free_Index)
 := (Rw.First_Free_Index  => 01, -- @ variable name number string char
     Rw.Left_Parenthesis  => 02, -- (
     Rw.Right_Parenthesis => 03, -- )
     Rw.Comma             => 04, -- ,
     Rw.Quotation_Mark    => 05, -- '
     Rw.Vertical_Line     => 06, -- |
     Rw.Double_Dot        => 07, -- ..
     Rw.Arrow             => 08, -- =>
     Rw.Dot               => 09, -- .
     Rw.N_With            => 10, -- with
     Rw.N_Null            => 11, -- null
     Rw.N_Record          => 12, -- record
     Rw.N_New             => 13, -- new
     Rw.N_All             => 14, -- all
     Rw.Plus_Sign
     | Rw.Minus
     | Rw.N_Not
     | Rw.N_Abs  => 15, -- + - not abs
     Rw.N_Others => 16, -- others
     Rw.Ampersand
     | Rw.Divide
     | Rw.Star
     | Rw.Exponentiate
     | Rw.Equals_Sign
     | Rw.Inequality
     | Rw.Greater_Than
     | Rw.Less_Than
     | Rw.Greater_Than_Or_Equal
     | Rw.Less_Than_Or_Equal => 17, --  & / * ** = /= > < >= <=
     Rw.N_Or
     | Rw.N_And
     | Rw.N_Xor => 18, -- or and xor
     Rw.N_Else
     | Rw.N_Then => 19, -- else then   (to be used for "or else" , "and then")
     Rw.N_In     => 20, -- in
     Rw.N_Rem
     | Rw.N_Mod => 21, -- rem mod  ( cannot be first)
     others     => 22);--miscellaneous
 
 --|
 --|                                 ||
 --|                                 |  Successful_End      |
 --|                                 |______________________|
 --|
 Successful_End : constant Truth_Table_Type(Look_Up_Table_Index_Type'Range)
 := (Rw.N_Delta
     | Rw.N_Digits
     | Rw.N_Do
     | Rw.N_Else
     | Rw.N_Elsif
     | Rw.N_Is
     | Rw.N_Loop
     | Rw.N_Of
     | Rw.N_Range
     | Rw.N_Renames
     | Rw.N_Return
     | Rw.N_Then
     | Rw.N_When
     | Rw.N_With
     | Rw.Semicolon
     | Rw.Assignment => True,
     others          => False);
 --|
 --|                                 ||
 --|                                 |  Exit_State_Ok       |
 --|                                 |______________________|
 --|
 Exit_State_Ok : constant State_Table_Type(State_Type'Range)
 := (Ok_State => True,
     03       => True, --  )
     05       => True, -- variable name
     others   => False);
 --|
 --|                                 ||
 --|                                 |  Depth_Zero          |
 --|                                 |______________________|
 --|
 Depth_Zero : constant Command_Table_Type(Command_Type'Range)
 := (08     => True, --    =>
     06     => True, --    |
     others => False);
 --|
 --|                                 ||
 --|                                 |  STATE/COMMAND table |
 --|                                 |______________________|
 --|
 --|
 --|
 --|
 State_Command_Matrix : constant State_Command_Matrix_Type(State_Type'Range, Command_Type'Range)
 --
 --        |||||||||||||||||||||||
 --        |01 |02 |03 |04 |05 |06 |07 |08 |09 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |20 |21 |22 |
 --        |@|||||||   |   |||||| ||o|o|| | |m|
 --        |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   | p | r | e |   | a | i |
 --        | n |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   | e |   | l |   | b | s |
 --        | u |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   | r |a  | s |   | s | c |
 --        | m |   |   |   |   |   |   |   |   |   |   | r |   |   |   | o | a |n  | e |   |   | e |
 --        | c |   |   |   |   |   |   |   |   |   |   | e |   |   |   | t | t |d  |   |   | m | l |
 --        | h |   |   |   |   |   |   |   |   | w | n | c |   |   |   | h | o |   | t |   | o | l |
 --        | a |   |   |   |   |   |   |   |   | i | u | o | n | a |   | e | r |   | h |   | d | a |   in use
 --        | s |   |   |   |   |   |   |   |   | t | l | r | e | l | + | r |   |   | e | I |   | n |
 --        | t | ( | ) | , | ' | | |.. | =>| . | h | l | d | w | l | - | s |   |   | n | N |   | e |      |
 --        || |||||||||||||||||||||      |
 --        |01 |02 |03 |04 |05 |06 |07 |08 |09 |10 |11 |12 |13 |14 |15 |16 |17 |18 |19 |20 |21 |   |      V
 := (00 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 00 *
     01 => (05, 03, 00, 00, 00, 00, 00, 00, 00, 07, 05, 00, 04, 00, 04, 05, 00, 00, 00, 01, 13, 00), -- 01 *
     02 => (00, 00, 02, 00, 00, 00, 00, 00, 05, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 02 *
     03 => (00, 03, 02, 01, 06, 00, 01, 00, 06, 07, 00, 00, 00, 00, 04, 00, 01, 11, 00, 01, 13, 00), -- 03 *
     04 => (05, 03, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 01, 13, 00), -- 04 *
 
     05 => (00, 03, 02, 01, 06, 01, 01, 01, 06, 07, 00, 09, 00, 00, 10, 00, 01, 11, 00, 01, 13, 00), -- 05 *
     06 => (05, 03, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 05, 00, 00, 00, 00, 00, 00, 00, 00), -- 06 *
     07 => (05, 00, 00, 00, 00, 00, 00, 00, 00, 00, 08, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 07 *
     08 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 09, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 08 *
     09 => (00, 00, 02, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 09 *
 
     10 => (05, 03, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 01, 00, 00, 01, 00, 00), -- 10 *
     11 => (05, 03, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 04, 00, 00, 00, 12, 00, 00, 00), -- 11 *
     12 => (05, 03, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 04, 00, 05, 00, 00, 00, 00, 00), -- 12 *
     13 => (05, 03, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 13 *
     14 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 14
 
     15 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 15
     16 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 16
     17 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 17
     18 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 18
     19 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 19
 
     20 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 20
     21 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 21
     22 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 22
     23 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 23
     24 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 24
 
     25 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 25
     26 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 26
     27 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 27
     28 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 28
     29 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 29
 
     30 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 30
     31 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 31
     32 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 32
     33 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00), -- 33
     34 => (00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00));-- 34
 --|
 --| no to be created at every recursive call, one copy is enough
 --|
 Token_Number         : Integer;
 Exit_When_Depth_Zero : Boolean;
 End_Token_Found      : Boolean;
 Next_Number          : Integer;
 Left_Par_Count       : Integer := 0;
 Debug_Flag           : Boolean := False;
 end Name_Test;
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
