 --|
 --
 --
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEEEEEEE
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEE
 --    P               A          A    R R                       S    E
 --    P               A          A    R  R                      S    E
 --    P               A          A    R   R                     S    E
 --    P               A          A    R     R                   S    E
 --    P               A          A    R       R                 S    E
 --    P               A          A    R         R     SSSSSSSSSSS    EEEEEEEEEEEE
 --
 --
 --
 --
 --     Copyright (C)  2002  Daniel Gaudry (Daniel.Gaudry@wanadoo.fr)
 --     9 av calmels
 --     92270 bois colombes  france
 --     Home +33147862334
 --     Cell +33664195009
 --     Work +33148575063
 --     Fax  +33148571106
 --
 --     This library is free software; you can redistribute it and/or
 --     modify it under the terms of the GNU Library General Public
 --     License as published by the Free Software Foundation; either
 --     version 2 of the License, or (at your option) any later version.
 --
 --     This library is distributed in the hope that it will be useful,
 --     but WITHOUT ANY WARRANTY; without even the implied warranty of
 --     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 --     Library General Public License for more details.
 --
 --     You should have received a copy of the GNU Library General Public
 --     License along with this library; if not, write to the Free
 --     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 --     MA 02111-1307, USA
 --|
 --|
 --|
 --|
 --| version 1.0 december 1998 original coding
 
 
 package Poly_Dic_Tree
 is
 --       COMMENTS will be appreciated
 --       Daniel.Gaudry@wanadoo.fr
 
 
 No_Such_A_Word : exception;
 Wrong_Tag_Type : exception;
 
 ----------------------------------------------------------------
 --|             CHARACTER IN DICTIONARY TREE TYPE DECLARATION  |
 --|                                                            |
 --|  T O   B E   M O D I F I E D   A C C O R D I N G           |
 --|           T O   Y O U R   N E E D S                        |
 --| currently set from underscore to z NO CAPITAL LETTER !!    |
 ---------------------------------------------------------------|
 
-- ACCEPTABLE CHARACTERS IN THE ASCII TABLE FROM _ TO z
 
 subtype Character_Value_Type is Character range '_' .. 'z';
 
 --|
 --|             TAGGED TYPE DECLARATION
 --|
 type Tagged_Data_Type is abstract tagged null record;
 type Tagged_Data_Access_Type is access all Tagged_Data_Type'Class;
 --|
 --|             NODE TYPE DECLARATION
 --|
 type Dictionary_Tree_Type is limited private;
 --|
 --|              ||
 --|              | Clear_Dictionary  |
 --|              |___________________|
 --|
 procedure Clear_Dictionary(The_Dictionary_Tree : in out Dictionary_Tree_Type);
 --|
 --|              ||
 --|              | Add_To_Dictionary |
 --|              |___________________|
 --|
 procedure Add_To_Dictionary(The_Record             : in     Tagged_Data_Type'Class;
                             In_The_Dictionary_Tree : in out Dictionary_Tree_Type;
                             With_The_Label         : in     String);
 --|
 --|              ||
 --|              | Print_Dictionary |
 --|              |__________________|
 --|
 procedure Print_Dictionary(Dictionary_Tree : in     Dictionary_Tree_Type);
 --|
 --|              ||
 --|              | Stat |
 --|              |______|
 --|
 procedure Stat(How_Many_Nodes_Created :    out Long_Integer);
 --|
 --|              ||
 --|              | Get_Record_Access_From_Label |
 --|              |______________________________|
 --|
 procedure Get_Record_Access_From_Label(From_The_Dictionary_Tree : in out Dictionary_Tree_Type;
                                        And_The_Label            : in     String;
                                        Tagged_Data_Access       :    out Tagged_Data_Access_Type);
 
 ------------------------------------------------
 ---      P A S S I V E   I T E R A T O R     ---
 ------------------------------------------------
 
 generic
 --|
 --|              ||
 --|              | Iterate |
 --|              |_________|
 --|
 with procedure Iterate(The_Node_Access : in out Tagged_Data_Access_Type;
                        Next_One        : in out Boolean;
                        The_Label       : in out String);
 --|
 --|              ||
 --|              | Visit_All |
 --|              |___________|
 --|
 procedure Visit_All(Dictionary_Tree : in     Dictionary_Tree_Type);
 
 ------------------------------------------------
 ------------------------------------------------
 ------------------------------------------------
 
 private
 
 type Node_Access_Type is access Dictionary_Tree_Type;
 
 type Node_Access_Array_Type is array (Character_Value_Type'Range) of Node_Access_Type;
 
 type Dictionary_Tree_Type is
 record
 End_Of_Word        : Boolean                 := False;
 Node_Access_Array  : Node_Access_Array_Type  := (others => null);
 Character_Value    : Character_Value_Type    := Character_Value_Type'First;
 Tagged_Data_Access : Tagged_Data_Access_Type := null;
 end record;
 
 Local_Hash_Value : Integer      := 0;
 Index            : Integer      := 0;
 Node_Num         : Long_Integer := 0;
 
 end Poly_Dic_Tree;
 
 
 
 
 
