 --
 --
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEEEEEEE
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEE
 --    P               A          A    R R                       S    E
 --    P               A          A    R  R                      S    E
 --    P               A          A    R   R                     S    E
 --    P               A          A    R     R                   S    E
 --    P               A          A    R       R                 S    E
 --    P               A          A    R         R     SSSSSSSSSSS    EEEEEEEEEEEE
 --
 --
 --
 --
 --     Copyright (C)  2002  Daniel Gaudry (Daniel.Gaudry@wanadoo.fr)
 --     9 av calmels
 --     92270 bois colombes  france
 --     Home +33147862334
 --     Cell +33664195009
 --     Work +33148575063
 --     Fax  +33148571106
 --
 --     This library is free software; you can redistribute it and/or
 --     modify it under the terms of the GNU Library General Public
 --     License as published by the Free Software Foundation; either
 --     version 2 of the License, or (at your option) any later version.
 --
 --     This library is distributed in the hope that it will be useful,
 --     but WITHOUT ANY WARRANTY; without even the implied warranty of
 --     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 --     Library General Public License for more details.
 --
 --     You should have received a copy of the GNU Library General Public
 --     License along with this library; if not, write to the Free
 --     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 --     MA 02111-1307, USA
 
 --Version 1.00 May 2nd, 2002
 
 
 with Ada.Strings.Maps;
 with Ada.Strings.Maps.Constants;
 with Ada.Strings.Fixed;
 with Ada.Strings;
 with Ada.Text_Io;
 
 
 
 package body Rt
 is
 --|
 --|
 --|
 --|                                 ||
 --|                                 |      Trace           |
 --|                                 |______________________|
 --|
    procedure Trace(Text : in     String)
    is
    begin
    Ada.Text_Io.Put_Line(Text);
    end Trace;
 --|
 --|              ||
 --|              | move_and_left_justify  |
 --|              |________________________|
 --|
    procedure Move_And_Left_Justify(From : in     String;
                                    To   : in out String)
    is
    begin
    if To'Length < From'Length
       then
       To := From(From'First .. From'First + To'Length - 1);
       else
       To := From & (From'Length + 1 .. To'Length => ' ');
    end if;
    end Move_And_Left_Justify;
 
 --|
 --|              ||
 --|              | Iteration_Over |
 --|              |________________|
 --|
    function Iteration_Over
    return Boolean
    is
    begin
    return Done;
    end Iteration_Over;
 
 --|
 --|              ||
 --|              | READ       |
 --|              |____________|
 --|
    procedure Read(Code_State : in out Code_State_Type)
 
    is
    --|
    --|              ||
    --|              | Update_Access |
    --|              |_______________|
    --|
       procedure Update_Access(With_What : in     String)
       is
       begin
       Code_State.Current_Index := 1 + Code_State.Current_Index;
       Code_State.Next_Index    := 1 + Code_State.Next_Index;
       Code_State.After_Index   := 1 + Code_State.After_Index;
 
 
       Circular_Buffer(Last_Index_Loaded) := new String'(Ada.Strings.Fixed.Trim(Source => With_What,
                                                                                Side   => Ada.Strings.Both));
       Last_Index_Loaded := 1 + Last_Index_Loaded;
       --|
       --|
       --|       d e b u g   m e s s a g e      
       --|_____________________________________________________________________________
       --|
       if Circular_Buffer(Code_State.Current_Index) /= null
          and then Circular_Buffer(Code_State.Next_Index) /= null
          and then Circular_Buffer(Code_State.After_Index) /= null
          then
          Trace(Long_Integer'Image(Line_Number)
                & ">>>>>Update_Access >>>>>CURRENT >"
                & Circular_Buffer(Code_State.Current_Index).all
                & "<NEXT>"
                & Circular_Buffer(Code_State.Next_Index).all
                & "<AFTER>"
                & Circular_Buffer(Code_State.After_Index).all
                & "<"
                & Index_Type'Image(Code_State.Current_Index)
                & Index_Type'Image(Code_State.Next_Index)
                & Index_Type'Image(Code_State.After_Index)
                & Index_Type'Image(Last_Index_Loaded));
       end if;
       --|
       --|
       --|    e n d   d e b u g   m e s s a g e  
       --|_____________________________________________________________________________
       --|
       end Update_Access;
 
    Previous_Start_At : Integer;
    Ok_To_Exit        : Boolean := False;
    use type Ae_Global.Elements_Type;
    --|
    --|              ||
    --|              | Read_Line  |
    --|              |____________|
    --|
       procedure Read_Line
       is
       begin
       --|
       --| READ LINE FROM FILE
       --|
       Ring_Elem := (others => ' ');
       Read :
       declare
       Dummy      : Natural;
       Map_String : constant String(1 .. 2) := (1 => Ascii.Vt, 2 => Ascii.Ht);
 
       My_Mapping : constant Ada.Strings.Maps.Character_Mapping := Ada.Strings.Maps.To_Mapping(From => Map_String,
                                                                                               To   => "  ");
 
          begin
          Line_Number := 1 + Line_Number;
          --|
          --|get line
          --|
          Ring_Elem := (others => ' ');
 
          Ada.Text_Io.Get_Line(File => Code_File,
                               Item => Ring_Elem,
                               Last => Dummy);
          --|
          --|                          loop to  map all ht vt to space in the string
          --|
          for K in Ring_Elem'Range loop
             --|
             --|                             map the current character
             --|
             Ring_Elem(K) := Ada.Strings.Maps.Value(Map     => My_Mapping,
                                                    Element => Ring_Elem(K));
          end loop;
          --|
          --| map to lower case character for reserved words recognition
          --|
          Ring_Elem := Ada.Strings.Fixed.Translate(Source  => Ring_Elem,
                                                   Mapping => Ada.Strings.Maps.Constants.Lower_Case_Map);
          --|
          --|
          --|       d e b u g   m e s s a g e      
          --|_____________________________________________________________________________
          --|
          Trace(Long_Integer'Image(Line_Number)
                               & "**"
                               & Ada.Strings.Fixed.Trim(Source => Ring_Elem,
                                                        Side   => Ada.Strings.Both)
                               & "**");
 
          if Circular_Buffer(Code_State.Current_Index) /= null
             and then Circular_Buffer(Code_State.Next_Index) /= null
             and then Circular_Buffer(Code_State.After_Index) /= null
             then
             Trace(Long_Integer'Image(Line_Number)
                   & ">>>>>Update_Access >>>>>CURRENT >"
                   & Circular_Buffer(Code_State.Current_Index).all
                   & "<NEXT>"
                   & Circular_Buffer(Code_State.Next_Index).all
                   & "<AFTER>"
                   & Circular_Buffer(Code_State.After_Index).all
                   & "<");
          end if;
          --|
          --|
          --|    e n d   d e b u g   m e s s a g e  
          --|_____________________________________________________________________________
          --|
          exception
          when Ada.Text_Io.End_Error =>
          Done         := True;
          Line_Is_Done := True;
          end Read;
       Line_Is_Done := False;
       end Read_Line;
    --|
    --|              ||
    --|              | Parse_Line |
    --|              |____________|
    --|
       procedure Parse_Line
       is
       begin
       --|
       --|parse the line
       --|
       Ae_20.Parse(The_Line           => Ada.Strings.Fixed.Trim
                   (Source            => Ring_Elem,
                    Side              => Ada.Strings.Right) & "  ",
                   How_Many_Words     => Number_Of_Tokens,
                   Token_Record_Array => Token_Record_Array);
       --|init
       --|
       Tokens_Start_At := Ada.Strings.Fixed.Index_Non_Blank(Source => Ring_Elem,
                                                            Going  => Ada.Strings.Forward);
       Current_Token_Number := 1;
       end Parse_Line;
 
    --|
    --|              ||
    --|              | READ       |
    --|              |____________|
    --|
    Token_Kind  : Ae_Global.Kind_Of_Token_Type;
    Delta_Index : Integer;
    begin
    --|
    --|
    --|       d e b u g   m e s s a g e      
    --|_____________________________________________________________________________
    --|
 
    if Last_Index_Loaded > Code_State.After_Index
       then
       Delta_Index := Integer(Last_Index_Loaded) - Integer(Code_State.After_Index);
       else
       Delta_Index := Integer(Index_Type'Last) + Integer(Last_Index_Loaded) - Integer(Code_State.After_Index);
    end if;
    if Init_Done and then Delta_Index > 1
       then
       Code_State.Current_Index := 1 + Code_State.Current_Index;
       Code_State.Next_Index    := 1 + Code_State.Next_Index;
       Code_State.After_Index   := 1 + Code_State.After_Index;
       --|
       --|
       --|       d e b u g   m e s s a g e      
       --|_____________________________________________________________________________
       --|
       Trace(Long_Integer'Image(Line_Number)
             & ">>>>> NO READ >>>>>CURRENT >"
             & Circular_Buffer(Code_State.Current_Index).all
             & "<NEXT>"
             & Circular_Buffer(Code_State.Next_Index).all
             & "<AFTER>"
             & Circular_Buffer(Code_State.After_Index).all
             & "<"
             & Index_Type'Image(Code_State.Current_Index)
             & Index_Type'Image(Code_State.Next_Index)
             & Index_Type'Image(Code_State.After_Index)
             & Index_Type'Image(Last_Index_Loaded));
       --|
       --|
       --|    e n d   d e b u g   m e s s a g e  
       --|_____________________________________________________________________________
       --|
       return;
    end if;
    --end if;
    --|
    --| over ???
    --|
    if Done
       then
       Update_Access(With_What => "");
       return;
    end if;
    --|
    --| loop until valid token found
    --|
    loop
       Current_Token_Number := 1 + Current_Token_Number;
       Line_Is_Done         := Current_Token_Number > Number_Of_Tokens;
       --|
       --| loop until non empty line is parsed
       --|
       loop
          if Line_Is_Done and then not Done
             then
             Read_Line;
             Parse_Line;
          end if;
          exit when Tokens_Start_At /= 0 or Done;
          --|
          --| loop until non empty line
          --|
          Line_Is_Done := True;
       end loop;
       --|
       --| empty line
       --|
       if Tokens_Start_At = 0
          then
          Done := True;
          exit;
       end if;
       --|
       --|seek all
       --|
       if Ring_Elem(Tokens_Start_At) = ' '
          then
          Tokens_Start_At := Tokens_Start_At + 1;
       end if;
       --|
       --|
       Previous_Start_At := Tokens_Start_At;
       --|
       --|
       --|
       Token_Kind      := Token_Record_Array(Current_Token_Number).Kind_Of_Token;
       Tokens_Start_At := Integer(Token_Record_Array(Current_Token_Number).Token_Ends_At) + 1;
       --|
       --|
       --|
       if Token_Kind = Ae_Global.Comment
          then
          Current_Token_Number := 1 + Number_Of_Tokens;-- flag line is done
          else
          Ok_To_Exit := (Token_Kind = Ae_Global.Attribute) or else
          (Token_Kind = Ae_Global.String) or else
          (Token_Kind = Ae_Global.Separator) or else
          (Token_Kind = Ae_Global.Variable_Name) or else
          (Token_Kind = Ae_Global.Reserved_Word) or else
          (Token_Kind = Ae_Global.Number) or else Done;
          --|
          --|
          Ok_To_Exit := Ok_To_Exit and then (Ring_Elem(Previous_Start_At .. Tokens_Start_At + 1) /= "  ");
          --|
          --|
          --|       d e b u g   m e s s a g e      
          --|_____________________________________________________________________________
          --|
          Trace("@token read@"
                & Ring_Elem(Previous_Start_At .. Tokens_Start_At - 1)
                & "@ "
                & Boolean'Image(Ok_To_Exit)
                & " "
                & Ae_Global.Elements_Type'Image(Token_Kind));
          --|
          --|
          --|    e n d   d e b u g   m e s s a g e  
          --|_____________________________________________________________________________
          --|
          exit when Ok_To_Exit;
       end if;
    end loop;
    if Token_Kind = Ae_Global.Variable_Name
       or else Token_Kind = Ae_Global.Attribute
       then
       Update_Access(With_What => Identifier);-- identifier
 
       elsif Token_Kind = Ae_Global.Number
       then
       Update_Access(With_What => Number);-- number
 
       elsif Token_Kind = Ae_Global.String
       then
       Update_Access(With_What => Char_Or_String);-- string
 
       elsif Token_Kind = Ae_Global.Space
       then
       null;
       else
       Update_Access(With_What => Ring_Elem(Previous_Start_At .. Tokens_Start_At - 1));
    end if;
 
 
    end Read;
 --|
 --|              ||
 --|              | Current_Line_Number |
 --|              |_____________________|
 --|
    function Current_Line_Number return Long_Integer
    is
    begin
 
    return Line_Number;
 
    end Current_Line_Number;
 --|
 --|              ||
 --|              | Current_Line |
 --|              |______________|
 --|
    function Current_Line return String
    is
    begin
 
    return Ring_Elem;
 
    end Current_Line;
 --|
 --|              ||
 --|              | init               |
 --|              |____________________|
 --|
    procedure Init(Name               : in     String;
                   Current_Code_State :    out Code_State_Type)
    is
    begin
 
    Ada.Text_Io.Open(File => Code_File,
                     Mode => Ada.Text_Io.In_File,
                     Name => Name);
    --|
    --|init and load next_token
    --|
    Rt.Read(Current_Code_State);
    Rt.Read(Current_Code_State);
    Rt.Read(Current_Code_State);
    Init_Done := True;
    end Init;
 --|
 --|              ||
 --|              | Current_Token_Position |
 --|              |________________________|
 --|
    function Current_Token_Position return Natural
    is
    begin
 
    return Tokens_Start_At;
 
    end Current_Token_Position;
 --|
 --|              ||
 --|              | Current       |
 --|              |_______________|
 --|
    function Current(Code_State : in     Code_State_Type)
    return String_Access_Type
    is
    begin
    return Circular_Buffer(Code_State.Current_Index);
    end Current;
 --|
 --|              ||
 --|              | Next          |
 --|              |_______________|
 --|
    function Next(Code_State : in     Code_State_Type)
    return String_Access_Type
    is
    begin
    return Circular_Buffer(Code_State.Next_Index);
    end Next;
 --|
 --|              ||
 --|              | After         |
 --|              |_______________|
 --|
    function After(Code_State : in     Code_State_Type)
    return String_Access_Type
    is
    begin
    return Circular_Buffer(Code_State.After_Index);
    end After;
 --|
 --|              ||
 --|              | print      |
 --|              |____________|
 --|
    function Print(Code_State : in     Code_State_Type) return String
    is
    begin
    return (Index_Type'Image(Code_State.Current_Index)
            & Index_Type'Image(Code_State.Next_Index)
            & Index_Type'Image(Code_State.After_Index));
    end Print;
 end Rt;
 
 
 
 
 
 
 
 
 
 
 
 
