 --
 --
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEEEEEEE
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    P          P    A          A    R          R    S              E
 --    PPPPPPPPPPPP    AAAAAAAAAAAA    RRRRRRRRRRRR    SSSSSSSSSSS    EEEEEEE
 --    P               A          A    R R                       S    E
 --    P               A          A    R  R                      S    E
 --    P               A          A    R   R                     S    E
 --    P               A          A    R     R                   S    E
 --    P               A          A    R       R                 S    E
 --    P               A          A    R         R     SSSSSSSSSSS    EEEEEEEEEEEE
 --
 --
 --
 --
 --     Copyright (C)  2002  Daniel Gaudry (Daniel.Gaudry@wanadoo.fr)
 --     9 av calmels
 --     92270 bois colombes  france
 --     Home +33147862334
 --     Cell +33664195009
 --     Work +33148575063
 --     Fax  +33148571106
 --
 --     This library is free software; you can redistribute it and/or
 --     modify it under the terms of the GNU Library General Public
 --     License as published by the Free Software Foundation; either
 --     version 2 of the License, or (at your option) any later version.
 --
 --     This library is distributed in the hope that it will be useful,
 --     but WITHOUT ANY WARRANTY; without even the implied warranty of
 --     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 --     Library General Public License for more details.
 --
 --     You should have received a copy of the GNU Library General Public
 --     License along with this library; if not, write to the Free
 --     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 --     MA 02111-1307, USA
 --
 --Version 1.00 May 2nd, 2002
 
 
 with Ae_Global;
 with Ada.Text_Io;
 with Ae_20;
 
 
 package Rt
 is
 
 type Code_State_Type is private;
 
 Identifier     : constant String := "@";
 Number         : constant String := "";
 Char_Or_String : constant String := "";
 
 
 type String_Access_Type is access all String;
 --|
 --|
 --|
 --|                                 ||
 --|                                 |      Trace           |
 --|                                 |______________________|
 --|
 procedure Trace(Text : in     String);
 --|
 --|              ||
 --|              | init               |
 --|              |____________________|
 --|
 procedure Init(Name               : in     String;
                Current_Code_State :    out Code_State_Type);
 --|
 --|              ||
 --|              | Iteration_Over |
 --|              |________________|
 --|
 function Iteration_Over return Boolean;
 
 --|
 --|              ||
 --|              | Current_Line_Number |
 --|              |_____________________|
 --|
 function Current_Line_Number return Long_Integer;
 --|
 --|              ||
 --|              | Current_Line |
 --|              |______________|
 --|
 function Current_Line return String;
 --|
 --|              ||
 --|              | Current_Token_Position |
 --|              |________________________|
 --|
 function Current_Token_Position return Natural;
 --|
 --|              ||
 --|              | READ       |
 --|              |____________|
 --|
 procedure Read(Code_State : in out Code_State_Type);
 --|
 --|              ||
 --|              | Current       |
 --|              |_______________|
 --|
 function Current(Code_State : in     Code_State_Type) return String_Access_Type;
 --|
 --|              ||
 --|              | Next          |
 --|              |_______________|
 --|
 function Next(Code_State : in     Code_State_Type) return String_Access_Type;
 --|
 --|              ||
 --|              | After         |
 --|              |_______________|
 --|
 function After(Code_State : in     Code_State_Type) return String_Access_Type;
 --|
 --|              ||
 --|              | print      |
 --|              |____________|
 --|
 function Print(Code_State : in     Code_State_Type) return String;
 --|________________
 --|
 private
 --|________________
 --|
 --|
 --
 Init_Done : Boolean := False;
 type Index_Type is mod 10000;
 type Circular_Buffer_Type is array (Index_Type) of String_Access_Type;
 
 type Code_State_Type is
 record
    Current_Index : Index_Type := Index_Type'First;
    Next_Index    : Index_Type := Index_Type'First + 1;
    After_Index   : Index_Type := Index_Type'First + 2;
 end record;
 
 subtype Ring_Elem_Type is String(Ae_Global.Line_Length_Range_Type'Range);
 
 Last_Index_Loaded    : Index_Type                              := Index_Type'First + 3;
 Circular_Buffer      : Circular_Buffer_Type                    := (others => null);
 Line_Is_Done         : Boolean                                 := False;
 Current_Token_Number : Long_Integer                            := 1;
 Done                 : Boolean                                 := False;
 Ring_Elem            : Ring_Elem_Type                          := (others => ' ');
 Number_Of_Tokens     : Long_Integer                            := 0;
 Tokens_Start_At      : Natural                                 := 1;
 Token_Ends_At        : Long_Integer                            := 1;
 Line_Number          : Long_Integer                            := 0;
 Token_Record_Array   : Ae_20.Token_Record_Array_Type(1 .. 111) := (others       =>
                                                                  (Token_Ends_At => 1,
                                                                   Kind_Of_Token => Ae_Global.Kind_Of_Token_Type'First));
 Code_File : Ada.Text_Io.File_Type;
 end Rt;
 
 
 
 
 
 
